@@ -43,7 +43,7 @@ class ImageLoaderMachO : public ImageLoader {
 public:
 										ImageLoaderMachO(const char* path, int fd, const uint8_t firstPage[4096], uint64_t offset, uint64_t len, const struct stat& info, const LinkContext& context);
 										ImageLoaderMachO(const char* moduleName, const struct mach_header* mh, uint64_t len, const LinkContext& context);
-	virtual								~ImageLoaderMachO()  {}
+	virtual								~ImageLoaderMachO();
 
 	const char*							getInstallPath() const;
 	virtual void*						getMain() const;
@@ -107,13 +107,14 @@ private:
 
 			void		init();
 			void		parseLoadCmds();
+			uintptr_t	bindIndirectSymbol(uintptr_t* ptrToBind, const struct macho_section* sect, const char* symbolName, uintptr_t targetAddr, ImageLoader* targetImage, const LinkContext& context);
 			void		doBindIndirectSymbolPointers(const LinkContext& context, BindingLaziness bindness, bool onlyCoalescedSymbols);
 			void		doBindExternalRelocations(const LinkContext& context, bool onlyCoalescedSymbols);
 			uintptr_t	resolveUndefined(const LinkContext& context, const struct macho_nlist* symbol, bool twoLevel, ImageLoader **foundIn);
 			uintptr_t	getRelocBase();
 			void		doImageInit(const LinkContext& context);
 			void		doModInitFunctions(const LinkContext& context);
-			void		setupLazyPointerHandler();
+			void		setupLazyPointerHandler(const LinkContext& context);
 			void		applyPrebindingToDATA(uint8_t* fileToPrebind);
 			void		applyPrebindingToLoadCommands(const LinkContext& context, uint8_t* fileToPrebind, time_t timestamp);
 			void		applyPrebindingToLinkEdit(const LinkContext& context, uint8_t* fileToPrebind);
@@ -158,6 +159,7 @@ private:
 
 	static uint32_t					fgHintedBinaryTreeSearchs;
 	static uint32_t					fgUnhintedBinaryTreeSearchs;
+	static uint32_t					fgCountOfImagesWithWeakExports;
 };
 
 
